import sys

sys.path.append("~/handbook-ADPA")
from tqdm import tqdm
import datasets
from datasets import Dataset, DatasetDict
from math import log


def get_margin_logp_token(pa, pb):
    indices = set(pa['indices']) | set(pb['indices'])
    probs_a, probs_b = {i: min(pa['values']) for i in indices}, {i: min(pb['values']) for i in indices}
    for i, v in zip(pa['indices'], pa['values']):
        probs_a[i] = v
    for i, v in zip(pb['indices'], pb['values']):
        probs_b[i] = v
    log_probs_margin = {i: log(probs_a[i] / probs_b[i]) for i in pa['indices']}
    log_probs_margin = sorted(log_probs_margin.items(), reverse=True, key=lambda x: x[1])
    return {'indices': [i[0] for i in log_probs_margin], 'values': [i[1] for i in log_probs_margin]}


chosen_key = "chosen"
rejected_key = "student"
assert chosen_key in ['chosen', 'teacher']
assert rejected_key in ['rejected', 'student']
save_to = fr"precompute_dataset/h2o-1.8b-initial-base__{chosen_key}__{rejected_key}"

teacher_student_output_sft = datasets.load_from_disk("precompute/dpomix-h2o-1.8b-init-teacher-student-ouput-sft")
teacher_student_output_dpo = datasets.load_from_disk("precompute/dpomix-h2o-1.8b-init-teacher-student-ouput-dpo")
teacher_student_output_stu = datasets.load_from_disk("precompute/dpomix-h2o-1.8b-init-teacher-student-ouput-stu")

chosen_rejected_raw_sft = datasets.load_from_disk("precompute/dpomix-h2o-1.8b-chosen-rejected-sft")
chosen_rejected_raw_dpo = datasets.load_from_disk("precompute/dpomix-h2o-1.8b-chosen-rejected-dpo")
chosen_rejected_raw_stu = datasets.load_from_disk("precompute/dpomix-h2o-1.8b-chosen-rejected-stu")

new_dataset = {}
for split in ['train', 'test']:
    new_dataset_split = {
        # prompt
        "prompt": [],
        "prompt_input_ids": [],
        "prompt_attention_mask": [],

        # chosen
        "chosen": [],
        "chosen_input_ids": [],
        "chosen_attention_mask": [],
        "chosen_labels": [],
        "initial_chosen_logps": [],
        "reference_chosen_logps": [],
        "teacher_ref_chosen_logps": [],
        "reference_chosen_probs": [],
        "teacher_ref_chosen_probs": [],

        # rejected
        "rejected": [],
        "rejected_input_ids": [],
        "rejected_attention_mask": [],
        "rejected_labels": [],
        "initial_rejected_logps": [],
        "reference_rejected_logps": [],
        "teacher_ref_rejected_logps": [],
        "reference_rejected_probs": [],
        "teacher_ref_rejected_probs": [],

        "rejected_margin_logp_every": [],
    }
    for teacher_student_sft, \
        teacher_student_dpo, \
        teacher_student_stu, \
        chosen_rejected_sft, \
        chosen_rejected_dpo, \
        chosen_rejected_stu \
        in tqdm(zip(
            teacher_student_output_sft[split],
            teacher_student_output_dpo[split],
            teacher_student_output_stu[split],
            chosen_rejected_raw_sft[split],
            chosen_rejected_raw_dpo[split],
            chosen_rejected_raw_stu[split])):

        # check raw
        if chosen_key == "chosen":
            _chosen_labels_dpo = [i for i in chosen_rejected_dpo['chosen_labels'] if i != -100]
            _chosen_labels_sft = [i for i in chosen_rejected_sft['chosen_labels'] if i != -100]
            _chosen_labels_stu = [i for i in chosen_rejected_stu['chosen_labels'] if i != -100]
        else:  # chosen_key == "teacher"
            _chosen_labels_dpo = [i for i in teacher_student_dpo['chosen_labels'] if i != -100]
            _chosen_labels_sft = [i for i in teacher_student_sft['chosen_labels'] if i != -100]
            _chosen_labels_stu = [i for i in teacher_student_stu['chosen_labels'] if i != -100]

        if _chosen_labels_dpo != _chosen_labels_sft or _chosen_labels_sft != _chosen_labels_stu:
            continue

        if rejected_key == "rejected":
            _rejected_labels_dpo = [i for i in chosen_rejected_dpo['rejected_labels'] if i != -100]
            _rejected_labels_sft = [i for i in chosen_rejected_sft['rejected_labels'] if i != -100]
            _rejected_labels_stu = [i for i in chosen_rejected_stu['rejected_labels'] if i != -100]
        else:  # chosen_key == "student"
            _rejected_labels_dpo = [i for i in teacher_student_dpo['rejected_labels'] if i != -100]
            _rejected_labels_sft = [i for i in teacher_student_sft['rejected_labels'] if i != -100]
            _rejected_labels_stu = [i for i in teacher_student_stu['rejected_labels'] if i != -100]

        if _rejected_labels_dpo != _rejected_labels_sft or _rejected_labels_sft != _rejected_labels_stu:
            continue

        # collate dataset
        new_dataset_split['prompt'].append(chosen_rejected_sft['prompt'])
        new_dataset_split['prompt_input_ids'].append(chosen_rejected_sft['prompt_input_ids'])
        new_dataset_split['prompt_attention_mask'].append(chosen_rejected_sft['prompt_attention_mask'])

        if chosen_key == "chosen":
            new_dataset_split['chosen'].append(chosen_rejected_stu['chosen'])
            new_dataset_split['chosen_input_ids'].append(chosen_rejected_stu['chosen_input_ids'])
            new_dataset_split['chosen_attention_mask'].append(chosen_rejected_stu['chosen_attention_mask'])
            new_dataset_split['chosen_labels'].append(chosen_rejected_stu['chosen_labels'])
            new_dataset_split['initial_chosen_logps'].append(chosen_rejected_stu['reference_chosen_logps'])
            new_dataset_split['teacher_ref_chosen_logps'].append(chosen_rejected_sft['reference_chosen_logps'])
            new_dataset_split['reference_chosen_logps'].append(chosen_rejected_dpo['reference_chosen_logps'])
            new_dataset_split['reference_chosen_probs'].append(chosen_rejected_dpo['reference_chosen_probs'])
            new_dataset_split['teacher_ref_chosen_probs'].append(chosen_rejected_sft['reference_chosen_probs'])
        else:  # chosen_key == "teacher"
            new_dataset_split['chosen'].append(teacher_student_stu['chosen'])
            new_dataset_split['chosen_input_ids'].append(teacher_student_stu['chosen_input_ids'])
            new_dataset_split['chosen_attention_mask'].append(teacher_student_stu['chosen_attention_mask'])
            new_dataset_split['chosen_labels'].append(teacher_student_stu['chosen_labels'])
            new_dataset_split['initial_chosen_logps'].append(teacher_student_stu['reference_chosen_logps'])
            new_dataset_split['teacher_ref_chosen_logps'].append(teacher_student_sft['reference_chosen_logps'])
            new_dataset_split['reference_chosen_logps'].append(teacher_student_dpo['reference_chosen_logps'])
            new_dataset_split['reference_chosen_probs'].append(teacher_student_dpo['reference_chosen_probs'])
            new_dataset_split['teacher_ref_chosen_probs'].append(teacher_student_sft['reference_chosen_probs'])

        if rejected_key == "rejected":
            new_dataset_split['rejected'].append(chosen_rejected_stu['rejected'])
            new_dataset_split['rejected_input_ids'].append(chosen_rejected_stu['rejected_input_ids'])
            new_dataset_split['rejected_attention_mask'].append(chosen_rejected_stu['rejected_attention_mask'])
            new_dataset_split['rejected_labels'].append(chosen_rejected_stu['rejected_labels'])
            new_dataset_split['initial_rejected_logps'].append(chosen_rejected_stu['reference_rejected_logps'])
            new_dataset_split['teacher_ref_rejected_logps'].append(chosen_rejected_sft['reference_rejected_logps'])
            new_dataset_split['reference_rejected_logps'].append(chosen_rejected_dpo['reference_rejected_logps'])
            new_dataset_split['reference_rejected_probs'].append(chosen_rejected_dpo['reference_rejected_probs'])
            new_dataset_split['teacher_ref_rejected_probs'].append(chosen_rejected_sft['reference_rejected_probs'])
            new_dataset_split['rejected_margin_logp_every'].append(
                [get_margin_logp_token(pa, pb) for (pa, pb) in zip(
                    chosen_rejected_dpo['reference_rejected_probs'][0],
                    chosen_rejected_sft['reference_rejected_probs'][0])])

        else:  # rejected_key == "student"
            new_dataset_split['rejected'].append(teacher_student_stu['rejected'])
            new_dataset_split['rejected_input_ids'].append(teacher_student_stu['rejected_input_ids'])
            new_dataset_split['rejected_attention_mask'].append(teacher_student_stu['rejected_attention_mask'])
            new_dataset_split['rejected_labels'].append(teacher_student_stu['rejected_labels'])
            new_dataset_split['initial_rejected_logps'].append(teacher_student_stu['reference_rejected_logps'])
            new_dataset_split['teacher_ref_rejected_logps'].append(teacher_student_sft['reference_rejected_logps'])
            new_dataset_split['reference_rejected_logps'].append(teacher_student_dpo['reference_rejected_logps'])
            new_dataset_split['reference_rejected_probs'].append(teacher_student_dpo['reference_rejected_probs'])
            new_dataset_split['teacher_ref_rejected_probs'].append(teacher_student_sft['reference_rejected_probs'])
            new_dataset_split['rejected_margin_logp_every'].append(
                [get_margin_logp_token(pa, pb) for (pa, pb) in zip(
                    teacher_student_dpo['reference_rejected_probs'][0],
                    teacher_student_sft['reference_rejected_probs'][0])])
        pass
    new_dataset[split] = Dataset.from_dict(new_dataset_split)

new_dataset = DatasetDict(new_dataset)
new_dataset.save_to_disk(save_to)
print(new_dataset)
