#!/bin/bash
# 获取输入参数
MODEL_PATH=$1
GPU_IDS=$2
MODEL_NAME=$3

# 如果用户没有提供 MODEL_NAME，则使用模型路径的最后一层路径名
if [ -z "$MODEL_NAME" ]; then
  MODEL_NAME=$(basename ${MODEL_PATH})
fi

# 获取一个未被占用的端口号
get_free_port() {
  while :; do
    PORT=$(shuf -i 20000-60000 -n 1)
    ss -lpn | grep ":$PORT " > /dev/null
    if [ $? -ne 0 ]; then
      break
    fi
  done
  echo $PORT
}

# 获取未占用的端口号
PORT=$(get_free_port)

# 第一步：执行 vllm_generate.py
# 检查是否存在 ${MODEL_PATH}/gpt4_1106_preview_turn1_n_1.jsonl 文件
if [ ! -f "${MODEL_PATH}/datasets--argilla--dpo-mix-7k_turn1_n_1.jsonl" ]; then
  echo "文件不存在，开始执行 vllm_generate.py..."
  CUDA_VISIBLE_DEVICES=${GPU_IDS} python \
      /data/spgao/alignment-handbook-cs/validate_by_rm/vllm_generate.py \
          --data /data/spgao/datasets/datasets--argilla--dpo-mix-7k \
          --turns 1 --prompt_key chosen \
          --out_dir ${MODEL_PATH} \
          --model ${MODEL_PATH} \
          --max_length 2048 --top_k 1 --num_outputs 1 --dataset_split test
          fi

# 第二步：执行 post_generate_dpomix.py
python /data/spgao/alignment-handbook-cs/validate_by_rm/post_generate_dpomix.py \
 --input_file ${MODEL_PATH}/datasets--argilla--dpo-mix-7k_turn1_n_1.jsonl \
 --output_file /data/spgao/alignment-handbook-cs/validate_by_rm/dpomixtest/models/${MODEL_NAME}.json

# 第三步：执行 score_by_rm.py (fsfairx)
CUDA_VISIBLE_DEVICES=${GPU_IDS} NCCL_P2P_DISABLE=1 NCCL_IB_DISABLE=1 \
XDG_CACHE_HOME=/data/spgao/.cache/huggingface HF_HOME=/data/spgao/.cache/huggingface \
accelerate launch \
    --main_process_port ${PORT} \
    /data/spgao/alignment-handbook-cs/validate_by_rm/score_by_rm.py \
        --data /data/spgao/alignment-handbook-cs/validate_by_rm/dpomixtest/models/${MODEL_NAME}.json \
        --out /data/spgao/alignment-handbook-cs/validate_by_rm/dpomixtest/fsfairx/${MODEL_NAME}.json \
        --prompt_key instruction \
        --response_keys output \
        --batch_size 16 \
        --reward_model fsfairx

# 获取一个新的未占用的端口号
PORT=$(get_free_port)

# 第四步：执行 score_by_rm.py (armorm)
CUDA_VISIBLE_DEVICES=${GPU_IDS} NCCL_P2P_DISABLE=1 NCCL_IB_DISABLE=1 \
XDG_CACHE_HOME=/data/spgao/.cache/huggingface HF_HOME=/data/spgao/.cache/huggingface \
accelerate launch \
    --main_process_port ${PORT} \
    /data/spgao/alignment-handbook-cs/validate_by_rm/score_by_rm.py \
        --data /data/spgao/alignment-handbook-cs/validate_by_rm/dpomixtest/fsfairx/${MODEL_NAME}.json \
        --out /data/spgao/alignment-handbook-cs/validate_by_rm/dpomixtest/armorm/${MODEL_NAME}.json \
        --prompt_key instruction \
        --response_keys output \
        --batch_size 16 \
        --reward_model armorm

# 第五步：执行 get_mean_score.py
echo /data/spgao/alignment-handbook-cs/validate_by_rm/dpomixtest/armorm/${MODEL_NAME}.json